@@ -52,20 +52,21 @@ os.chdir(scriptDir)
 from wafw00f import __version__
 from wafw00f.lib.evillib import oururlparse, scrambledheader, waftoolsengine
 from wafw00f.manager import load_plugins
+from wafw00f.wafprio import wafdetectionsprio
 
-lackofart = """
+lackofart = r'''
                                  ^     ^
         _   __  _   ____ _   __  _    _   ____
        ///7/ /.' \ / __////7/ /,' \ ,' \ / __/
       | V V // o // _/ | V V // 0 // 0 // _/
       |_n_,'/_n_//_/   |_n_,' \_,' \_,'/_/
                                 <
-                                 ...'
+                                ...'
 
     WAFW00F - Web Application Firewall Detection Tool
 
     By Sandro Gauci && Wendel G. Henrique
-"""
+'''
 
 
 class WafW00F(waftoolsengine):
@@ -149,86 +150,73 @@ class WafW00F(waftoolsengine):
         ]
         # test if response for a path containing html tags with known evil strings
         # gives a different response from another containing invalid html tags
-        r = self.cleanhtml()
-        if r is None:
-            self.knowledge['generic']['reason'] = reasons[0]
-            self.knowledge['generic']['found'] = True
-            return True
-        cleanresponse, _tmp = r
-        r = self.xssstandard()
-        if r is None:
-            self.knowledge['generic']['reason'] = reasons[0]
-            self.knowledge['generic']['found'] = True
-            return True
-        xssresponse, _tmp = r
-        if xssresponse.status != cleanresponse.status:
-            self.log.info('Server returned a different response when a script tag was tried')
-            reason = reasons[2]
-            reason += '\r\n'
-            reason += 'Normal response code is "%s",' % cleanresponse.status
-            reason += ' while the response code to an attack is "%s"' % xssresponse.status
-            self.knowledge['generic']['reason'] = reason
-            self.knowledge['generic']['found'] = True
-            return True
-        r = self.cleanhtmlencoded()
-        cleanresponse, _tmp = r
-        r = self.xssstandardencoded()
-        if r is None:
-            self.knowledge['generic']['reason'] = reasons[0]
-            self.knowledge['generic']['found'] = True
-            return True
-        xssresponse, _tmp = r
-        if xssresponse.status != cleanresponse.status:
-            self.log.info('Server returned a different response when a script tag was tried')
-            reason = reasons[2]
-            reason += '\r\n'
-            reason += 'Normal response code is "%s",' % cleanresponse.status
-            reason += ' while the response code to an attack is "%s"' % xssresponse.status
-            self.knowledge['generic']['reason'] = reason
-            self.knowledge['generic']['found'] = True
-            return True
-        response, responsebody = self.normalrequest()
-        normalserver = response.getheader('Server')
-        for attack in self.attacks:
-            r = attack(self)
-            if r is None:
-                self.knowledge['generic']['reason'] = reasons[0]
+        try:
+            cleanresponse, _tmp = self._perform_and_check(self.cleanhtml)
+            xssresponse, _tmp = self._perform_and_check(self.xssstandard)
+            if xssresponse.status != cleanresponse.status:
+                self.log.info('Server returned a different response when a script tag was tried')
+                reason = reasons[2]
+                reason += '\r\n'
+                reason += 'Normal response code is "%s",' % cleanresponse.status
+                reason += ' while the response code to an attack is "%s"' % xssresponse.status
+                self.knowledge['generic']['reason'] = reason
                 self.knowledge['generic']['found'] = True
                 return True
-            response, responsebody = r
-            attackresponse_server = response.getheader('Server')
-            if attackresponse_server:
-                if attackresponse_server != normalserver:
-                    if (normalserver, attackresponse_server) in knownflops:
-                        return False
-                    self.log.info('Server header changed, WAF possibly detected')
-                    self.log.debug('attack response: %s' % attackresponse_server)
-                    self.log.debug('normal response: %s' % normalserver)
-                    reason = reasons[1]
-                    reason += '\r\nThe server header for a normal response is "%s",' % normalserver
-                    reason += ' while the server header a response to an attack is "%s.",' % attackresponse_server
-                    self.knowledge['generic']['reason'] = reason
-                    self.knowledge['generic']['found'] = True
-                    return True
-        for attack in self.wafdetectionsprio:
-            if self.wafdetections[attack](self) is None:
-                self.knowledge['generic']['reason'] = reasons[0]
+            cleanresponse, _tmp = self._perform_and_check(self.cleanhtmlencoded)
+            xssresponse, _tmp = self._perform_and_check(self.xssstandardencoded)
+            if xssresponse.status != cleanresponse.status:
+                self.log.info('Server returned a different response when a script tag was tried')
+                reason = reasons[2]
+                reason += '\r\n'
+                reason += 'Normal response code is "%s",' % cleanresponse.status
+                reason += ' while the response code to an attack is "%s"' % xssresponse.status
+                self.knowledge['generic']['reason'] = reason
                 self.knowledge['generic']['found'] = True
                 return True
-        for attack in self.attacks:
-            r = attack(self)
-            if r is None:
-                self.knowledge['generic']['reason'] = reasons[0]
-                self.knowledge['generic']['found'] = True
-                return True
-            response, responsebody = r
-            for h, v in response.getheaders():
-                if scrambledheader(h):
-                    self.knowledge['generic']['reason'] = reasons[4]
+            response, responsebody = self._perform_and_check(self.normalrequest)
+            normalserver = response.getheader('Server')
+            for attack in self.attacks:
+                response, responsebody = self._perform_and_check(lambda: attack(self))
+                attackresponse_server = response.getheader('Server')
+                if attackresponse_server:
+                    if attackresponse_server != normalserver:
+                        if (normalserver, attackresponse_server) in knownflops:
+                            return False
+                        self.log.info('Server header changed, WAF possibly detected')
+                        self.log.debug('attack response: %s' % attackresponse_server)
+                        self.log.debug('normal response: %s' % normalserver)
+                        reason = reasons[1]
+                        reason += '\r\nThe server header for a normal response is "%s",' % normalserver
+                        reason += ' while the server header a response to an attack is "%s.",' % attackresponse_server
+                        self.knowledge['generic']['reason'] = reason
+                        self.knowledge['generic']['found'] = True
+                        return True
+            for attack in wafdetectionsprio:
+                if self.wafdetections[attack](self) is None:
+                    self.knowledge['generic']['reason'] = reasons[0]
                     self.knowledge['generic']['found'] = True
                     return True
+            for attack in self.attacks:
+                response, responsebody = self._perform_and_check(lambda: attack(self))
+                for h, v in response.getheaders():
+                    if scrambledheader(h):
+                        self.knowledge['generic']['reason'] = reasons[4]
+                        self.knowledge['generic']['found'] = True
+                        return True
+        except RequestBlocked:
+            self.knowledge['generic']['reason'] = reasons[0]
+            self.knowledge['generic']['found'] = True
+            return True
+
         return False
 
+    def _perform_and_check(self, request_method):
+        r = request_method()
+        if r is None:
+            raise RequestBlocked()
+
+        return r
+
     def matchheader(self, headermatch, attack=False, ignorecase=True):
         import re
 
@@ -270,59 +258,8 @@ class WafW00F(waftoolsengine):
         """
         return self.matchheader(('set-cookie', match))
 
-    def isbeeware(self):
-        # disabled cause it was giving way too many false positives
-        # credit goes to Sebastien Gioria
-        detected = False
-        r = self.xssstandard()
-        if r is None:
-            return
-        response, responsebody = r
-        if (response.status != 200) or (response.reason == 'Forbidden'):
-            r = self.directorytraversal()
-            if r is None:
-                return
-            response, responsebody = r
-            if response.status == 403:
-                if response.reason == 'Forbidden':
-                    detected = True
-        return detected
-
-    def ismodsecuritypositive(self):
-        detected = False
-        self.normalrequest(usecache=False, cacheresponse=False)
-        randomfn = self.path + str(random.randrange(1000, 9999)) + '.html'
-        r = self.request(path=randomfn)
-        if r is None:
-            return
-        response, responsebody = r
-        if response.status != 302:
-            return False
-        randomfnnull = randomfn + '%00'
-        r = self.request(path=randomfnnull)
-        if r is None:
-            return
-        response, responsebody = r
-        if response.status == 404:
-            detected = True
-        return detected
-
     wafdetections = dict()
 
-    # easy ones
-    # lil bit more complex
-    #wafdetections['BeeWare'] = isbeeware
-    #wafdetections['ModSecurity (positive model)'] = ismodsecuritypositive removed for now
-    wafdetectionsprio = ['Profense', 'NetContinuum', 'Incapsula WAF', 'CloudFlare', 'NSFocus', 'Safedog',
-                         'Mission Control Application Shield', 'USP Secure Entry Server', 'Cisco ACE XML Gateway',
-                         'Barracuda Application Firewall', 'Art of Defence HyperGuard', 'BinarySec', 'Teros WAF',
-                         'F5 BIG-IP LTM', 'F5 BIG-IP APM', 'F5 BIG-IP ASM', 'F5 FirePass', 'F5 Trafficshield', 
-                         'InfoGuard Airlock', 'Citrix NetScaler',
-                         'Trustwave ModSecurity', 'IBM Web Application Security', 'IBM DataPower', 'DenyALL WAF',
-                         'Applicure dotDefender', 'Juniper WebApp Secure',  # removed for now 'ModSecurity (positive model)',
-                         'Microsoft URLScan', 'Aqtronix WebKnight', 
-                         'eEye Digital Security SecureIIS', 'Imperva SecureSphere', 'Microsoft ISA Server']
-
     plugin_dict = load_plugins()
     result_dict = {}
     for plugin_module in plugin_dict.values():
@@ -330,7 +267,12 @@ class WafW00F(waftoolsengine):
 
     def identwaf(self, findall=False):
         detected = list()
-        for wafvendor in self.wafdetectionsprio:
+
+        # Check for prioritized ones first, then check those added externally
+        checklist = wafdetectionsprio
+        checklist += list(set(self.wafdetections.keys()) - set(checklist))
+
+        for wafvendor in checklist:
             self.log.info('Checking for %s' % wafvendor)
             if self.wafdetections[wafvendor](self):
                 detected.append(wafvendor)
@@ -347,20 +289,22 @@ def calclogginglevel(verbosity):
         level = 0
     return level
 
+
 def getheaders(fn):
     headers = {}
-    fullfn = os.path.abspath(os.path.join(os.getcwd(),fn))
+    fullfn = os.path.abspath(os.path.join(os.getcwd(), fn))
     if not os.path.exists(fullfn):
-        logging.getLogger('wafw00f').critical('Headers file "%s" does not exist!'%fullfn)
+        logging.getLogger('wafw00f').critical('Headers file "%s" does not exist!' % fullfn)
         return
-    with open(fn,'r') as f:
-        for line in f.readlines():            
-            _t = line.split(':',2)
+    with open(fn, 'r') as f:
+        for line in f.readlines():
+            _t = line.split(':', 2)
             if len(_t) == 2:
-                h,v = map(lambda x: x.strip(),_t)
+                h, v = map(lambda x: x.strip(), _t)
                 headers[h] = v
     return headers
 
+
 def main():
     print(lackofart)
     parser = OptionParser(usage='%prog url1 [url2 [url3 ... ]]\r\nexample: %prog http://www.victim.org/')
@@ -378,15 +322,16 @@ def main():
                       default=False, help='Use an HTTP proxy to perform requests, example: http://hostname:8080, socks5://hostname:1080')
     parser.add_option('--version', '-V', dest='version', action='store_true',
                       default=False, help='Print out the version')
-    parser.add_option('--headersfile', '-H', dest='headersfile', action='store',
-                      default=None, help='Pass custom headers, for example to overwrite the default User-Agent string')
+    parser.add_option('--headersfile', '-H', dest='headersfile', 
+                      action='store', default=None, 
+                      help='Pass custom headers, for example to overwrite the default User-Agent string')
     options, args = parser.parse_args()
     logging.basicConfig(level=calclogginglevel(options.verbose))
     log = logging.getLogger()
     if options.list:
         print('Can test for these WAFs:\r\n')
         attacker = WafW00F(None)
-        print('\r\n'.join(attacker.wafdetectionsprio))
+        print('\r\n'.join(attacker.wafdetections.keys()))
         return
     if options.version:
         print('WAFW00F version %s' % __version__)
@@ -445,6 +390,10 @@ def main():
         print('Number of requests: %s' % attacker.requestnumber)
 
 
+class RequestBlocked(Exception):
+    pass
+
+
 if __name__ == '__main__':
     if sys.hexversion < 0x2060000:
         sys.stderr.write('Your version of python is way too old .. please update to 2.6 or later\r\n')
